<?php

class Quivers_Integration_Model_Observer
{
    public function taxUpdate($observer)
    {
        $log = new Quivers_Integration_Model_LogTracking();
        $startTime = microtime(true);
        $marketPlace = Mage::getStoreConfig('quivers_integration/settings/quivers_marketplace_id');
        $quiversSandbox = Mage::getStoreConfig('quivers_integration/settings/quivers_sandbox_enabled');
        if($quiversSandbox) {
            $taxUrl = 'https://api.quiverstest.com/v1/CustomerOrders/Validate';
        }
        else {
            $taxUrl = 'https://api.quivers.com/v1/CustomerOrders/Validate';   
        }
        if(!$marketPlace) {
            return;
        }
        $quote = Mage::getSingleton('checkout/session')->getQuote();
        $shippingMethods = array();
        $addresses = $quote->getAllShippingAddresses();

        $shippingFeesTotal =  0.0;
        $shippingFeePerItem = 0.0;
        $shipping_amt_diff = 0.0;
        foreach ($addresses as $address) {
            $id = $address->getCustomerAddressId();
            //Mage::log($address->toJson());
            //Mage::log(get_class_methods($address));

            $items = array();
            $productMapping = array();


            // if no item selected then no need to proceed further.
            if(count($quote->getAllItems()) <= 0) {
                //Mage::log(count($quote->getAllItems()));
                return $this;
            }

            // $shipping_address = $quote->getShippingAddress();
            // Mage::log("seee all shipping address");
            // Mage::log(count($quote->getAllShippingAddresses()));
            // if(!$shipping_address) {
            //     return $this;
            // }

            $country = $address->getCountryId();

            if(!$country) {
                return $this;
            }

            if(!$address->getRegionCode()) {
                return $this;
            }
            // Mage::log($address->getRegionCode());

            $shippingMethod = $address->getShippingMethod();
            $mshippingMethod = Mage::getSingleton('checkout/session')->getQuote()->getShippingAddress()->getShippingMethod();
            
            if($shippingMethod) {
                $shippingRateData = $address->getShippingRatesCollection()->getData();

                if(count($shippingRateData)>0) {
                    foreach ($shippingRateData as $rate) {
                        if(array_key_exists('code', $rate) && $rate["code"] == $mshippingMethod) {
                            if(array_key_exists('price', $rate)) {
                                $shippingFeesTotal += $rate["price"];
                            }
                        }
                    }
                }

                $totalItems = 0;
                foreach ($address->getAllVisibleItems() as $quoteItem) {
                    // $itemProduct = $quoteItem->getProduct();
                    // $unitPrice = (float)$itemProduct->getPrice();
                    $parent = $quoteItem->getParentItem();
                    if(!$parent) {
                        $totalItems = $totalItems +1;
                    }
                }
                if($totalItems ==0) {
                    return $this;
                }
                $shippingFeePerItem = (float)$shippingFeesTotal/$totalItems;
                $shippingFeePerItem = floatval(substr($shippingFeePerItem, 0, strpos($shippingFeePerItem, '.') + 3));
                $shipping_amt_diff = round($shippingFeesTotal - ($shippingFeePerItem * $totalItems), 2);

            }
            foreach ($address->getAllVisibleItems() as $quoteItem) {
                $parent = $quoteItem->getParentItem();
                if($parent) {
                    continue;
                }
                // Mage::log($quoteItem->toJson());
                $itemData = $quoteItem->getData();
                $itemProduct = $quoteItem->getProduct();
                $unitPrice = (float)$itemProduct->getPrice();
                // skip virtual products
                // Mage::log('unit price');
                // Mage::log("unit price".$unitPrice);

                $itemId = (int)$quoteItem->getId();

                $productMapping[$itemId] = $quoteItem;

                $itemName = $itemProduct->getName();
                $itemVariantName = $itemProduct->getSku();
                $options = array();
                $options = $quoteItem->getProduct()->getTypeInstance(true)->getOrderOptions($quoteItem->getProduct());
                $options_arr = array();
                if (isset($options['options']) && !empty($options['options'])) {
                    foreach ($options['options'] as $option) {
                        if(array_key_exists('label', $option)) {
                            $optionTitle = $option['label'];
                        }
                        if(array_key_exists('option_id', $option)) {
                            $optionId = $option['option_id'];
                        }
                        if(array_key_exists('value', $option)) {
                            $optionValue = $option['value'];
                        }
                        
                    array_push($options_arr, array(
                        "name" => $optionTitle,
                        "value" => array("name" => $optionValue)
                    ));

                    }
                }

                if(array_key_exists('base_price', $itemData)) {
                    $unitPrice = $itemData['base_price'];
                }

                $totalDiscount = $quoteItem->getDiscountAmount();

                $discounts = array();
                if($totalDiscount > 0 ) {
                    array_push($discounts,  array(
                        "name"=> "Total Discount",
                        "amount"=> -$totalDiscount,
                    ));
                }

                $shippingFees = array();
                if($shippingFeePerItem > 0 ) {
                    $shippingFees = array(
                        "name" => $shippingMethod,
                        "amount" => $shippingFeePerItem
                    );
                }

                if(count($shippingFees) > 0 ) {
                    $shippingFees = array($shippingFees);
                }

                $productId = (int)$quoteItem->getProductId();
                $obj = Mage::getModel('catalog/product');
                $_product = $obj->load($productId);
                $product_tax_category = $_product->getProductTaxCategory();

                array_push($items, array(
                "product" => array(
                    "name" => $itemName,
                    "refId" => $itemId,
                    "taxCode" => $product_tax_category,
                    "variant" => array(
                        "name" => $itemVariantName,
                        "options" => $options_arr
                    )
                ),

                "quantity" => $quoteItem->getQty(),
                "pricing" => array(
                        "unitPrice" => (float)$unitPrice,
                        "shippingFees" => $shippingFees,
                        "discounts"=> $discounts
                    )
                ));

            }
            $email = $quote->getShippingAddress()->getEmail();
            $taxData = array(
                "marketplaceId" => $marketPlace,
                "customer" => array(
                    "email" => $email
                ),
                "shippingAddress" => array(
                    "phoneNumber" => $address->getTelephone(),
                    "line1" => $address->getStreet1(),
                    "line2" => $address->getStreet2(),
                    "city" => $address->getCity(),
                    "postCode" => $address->getPostcode(),
                    "country" => $country,
                    "region" => $address->getRegionCode()
                ),
                "items" => $items
            );
            $order_details = ["Tax Data"=>$taxData];
            $log->session_start($order_details,date('r', $startTime)."UTC");

            foreach ($taxData["items"] as &$payload_item) {
                if (!$shipping_amt_diff) {
                    break;
                }
                $payload_item["pricing"]["shippingFees"][0]["amount"] += 0.01;
                $shipping_amt_diff -= 0.01;
            }
            unset($payload_item);

            // Mage::log($taxData);


            $ch = curl_init( $taxUrl );
            # Setup request to send json via POST.
            $payload = json_encode($taxData);
            // Mage::log($payload);
            curl_setopt( $ch, CURLOPT_POSTFIELDS, $payload );
            curl_setopt( $ch, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
            curl_setopt($ch, CURLOPT_HTTPHEADER,array("Expect:"));
            # Return response instead of printing.
            curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
            curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 9);
            curl_setopt( $ch, CURLOPT_TIMEOUT, 10);
            # Send request.
            $result = curl_exec($ch);
            $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            curl_close($ch);
            // Mage::log($result);

            if($httpcode==200) {
                $responseDecoded = json_decode($result, true);
                // Mage::log($responseDecoded);
                $totalTax = $responseDecoded["result"]["totals"]["taxes"]["additional"];
                $subtotal = $responseDecoded["result"]["totals"]["subtotal"];
                $totalAmount = $responseDecoded["result"]["totals"]["total"];
                foreach ($responseDecoded["result"]["items"] as $responseItems) {
                    $productId = (int)$responseItems['productRefId'];
                    $itemQuantity = $responseItems['quantity'];
                    $itemSubotal = $responseItems['pricing']['subtotal'] - $shippingFeePerItem;
                    $itemTotal = $responseItems['pricing']['total'] - $shippingFeePerItem;
                    $itemUnitPrice = $responseItems['pricing']['unitPrice'];

                    $quoteItem = $productMapping[$productId];
                    $quoteItemData = $quoteItem->getData();
                    $appliedTaxes = $responseItems['pricing']['taxes'];
                    if(count($responseItems['pricing']['taxes']) <= 0) {
                        continue;
                    }
                    $quoteItemData['price_incl_tax'] = $itemTotal/$itemQuantity;
                    $quoteItemData['base_price_incl_tax'] = $itemTotal/$itemQuantity;

                    $quoteItemData = $this->quoteItemTaxUpdate($quoteItemData, $appliedTaxes);
                    $quoteItemData['row_total_incl_tax'] = $itemTotal;
                    $quoteItem->setData($quoteItemData);


                    // $quoteItem->collectTotals();
                }
                // Mage::log(get_class_methods($address));
                // Mage::log($address->toJson());
                $addressData = $address->getData();
                $addressData['subtotal'] = $subtotal;
                $addressData['base_subtotal'] = $subtotal;
                $addressData['tax_amount'] = $totalTax;
                $addressData['base_tax_amount'] = $totalTax;
                $addressData['grand_total'] = $totalAmount;
                $addressData['base_grand_total'] = $totalAmount;
                $addressData['subtotal_incl_tax'] = $subtotal + $totalTax;
                $addressData['base_subtotal_total_incl_tax'] = $subtotal + $totalTax;
                $address->setGrandTotal($totalAmount);
                $address->setBaseGrandTotal($totalAmount);
                $address->setData($addressData);

                // set payment descriptor
                $session = Mage::getSingleton('checkout/session');
                $statementDescriptor = $responseDecoded["result"]["statementDescriptor"];
                if(strlen($statementDescriptor) > 10){
                    $statementDescriptor = substr($statementDescriptor, 0, 10);
                }
                $session->setPaymentStatementDescriptor($statementDescriptor);

                $endTime = microtime(true);
                $request =  ["tax_url" => $taxUrl, "tax_data" => $taxData];
                $log->validate_api_call($request, $responseDecoded, date('r', $startTime)."UTC",  date('r', $endTime)."UTC");
                $order_detail = ["Tax Data"=>$taxData, "Response"=>$responseDecoded];
                $log->session_end($order_detail,date('r', $endTime)."UTC");
            }
            else {
                if($quiversSandbox) {
                    $url = 'https://api.quiverstest.com/v1/countries';
                }
                else {
                    $url = 'https://api.quivers.com/v1/countries';
                }
                $curl = curl_init($url);
                curl_setopt($curl, CURLOPT_URL, $url);
                curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
                curl_setopt($curl, CURLOPT_HTTPHEADER,array("Expect:"));
                curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 4);
                curl_setopt($curl, CURLOPT_TIMEOUT, 5);
                $result = curl_exec($curl);
                $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
                curl_close($curl);
                $response = json_decode($result, true);
                $request =  ["url" => $url, "tax_data" => $taxData];

                foreach ($response['result'] as $country) {
                    if ($country['abbreviations']['two']==$address->getCountryId()) {
                        foreach ($country["regions"] as $region) {
                            if ($region['abbreviation']==$address->getRegionCode()){
                                $tax_rate = $region['maxTaxRate'];
                                break;
                            }
                        }
                        break;
                    }
                    else {
                        $tax_rate = 0;
                    }
                }

                $totalTax = 0; #(total tax)
                $subtotal = 0; #(order total excluding tax)
                $totalAmount = 0; #(order total including tax)

                foreach ($address->getAllVisibleItems() as $quoteItem) {
                    $parent = $quoteItem->getParentItem();
                    if($parent) {
                        continue;
                    }
                    $totalItemDiscount = $quoteItem->getDiscountAmount();
                    $itemQuantity = $quoteItem->getQty();
                    $itemProduct = $quoteItem->getProduct();
                    $itemUnitPrice = (float)$itemProduct->getPrice();
                    $itemData = $quoteItem->getData();
                    if(array_key_exists('base_price', $itemData)) {
                        $itemUnitPrice = $itemData['base_price'];
                    }
                    $taxableAmount = ($itemUnitPrice * $itemQuantity) - $totalItemDiscount;
                    $itemSubotal = $itemUnitPrice * $itemQuantity;
                    $tax_amount = $tax_rate * $taxableAmount;
                    $itemTotal = $taxableAmount + $tax_amount; #(item_price_with_tax - shipping_fee_per_item)
                    $quoteItemData = $quoteItem->getData();
                    $quoteItemData['price_incl_tax'] = $itemTotal/$itemQuantity;
                    $quoteItemData['base_price_incl_tax'] = $itemTotal/$itemQuantity;
                    $quoteItemData['tax_amount'] = $tax_amount;
                    $quoteItemData['base_tax_amount'] = $tax_amount;
                    $quoteItemData['tax_percent'] = $tax_rate * 100;
                    $quoteItemData['row_total_incl_tax'] = $itemTotal;
                    $quoteItem->setData($quoteItemData);

                    $totalTax = $totalTax + $tax_amount;
                    $subtotal = $subtotal + $itemSubotal;
                    $totalAmount = $totalAmount + $itemTotal;

                    // $quoteItem->collectTotals();
                }
                // Mage::log($address->toJson());
                $totalAmount = $totalAmount + $shippingFeesTotal;
                $addressData = $address->getData();
                $addressData['subtotal'] = $subtotal;
                $addressData['base_subtotal'] = $subtotal;
                $addressData['tax_amount'] = $totalTax;
                $addressData['base_tax_amount'] = $totalTax;
                $addressData['grand_total'] = $totalAmount;
                $addressData['base_grand_total'] = $totalAmount;
                $addressData['subtotal_incl_tax'] = $subtotal + $totalTax;
                $addressData['base_subtotal_total_incl_tax'] = $subtotal + $totalTax;
                $address->setGrandTotal($totalAmount);
                $address->setBaseGrandTotal($totalAmount);
                $address->setData($addressData);
                $endTime = microtime(true);
                $log->countries_api_call($request, $response, date('r', $startTime)."UTC",  date('r', $endTime)."UTC");
                $order_detail = ["Tax Data"=>$taxData, "Response"=>$response];
                $log->session_end($order_detail,date('r', $endTime)."UTC");
            }

        }
    }
    public function quoteItemTaxUpdate($quoteItemData, $appliedTaxes) {
        $quoteItemData['tax_amount'] = 0;
        $quoteItemData['base_tax_amount'] = 0;
        $quoteItemData['tax_percent'] = 0;
        foreach ($appliedTaxes as $appliedTax)
            $quoteItemData['tax_amount'] = $quoteItemData['tax_amount'] + $appliedTax['amount'];
            $quoteItemData['base_tax_amount'] = $quoteItemData['base_tax_amount'] + $appliedTax['amount'];
            $quoteItemData['tax_percent'] = $quoteItemData['tax_percent'] + $appliedTax['rate'];
        $quoteItemData['tax_percent'] = $quoteItemData['tax_percent'] * 100;
        return $quoteItemData;
    }    
}