<?php

require_once BP.DS.'lib'.DS.'stripe-php'.DS.'init.php';
// require_once(BP.DS."lib/stripe-php/lib/Stripe.php");

class Quivers_Integration_Model_Payment extends Mage_Payment_Model_Method_Cc {

	protected $_code	=	'quivers_integration';
	protected $_formBlockType = 'quivers_integration/form_stripepaymentmethod';
    // protected $_infoBlockType = 'quivers_integration/info_stripepaymentinfo';
	
    protected $_canAuthorize                = true;

    protected $_minOrderTotal = 0.5;
    
    public $stripe;

	public function __construct()
    {
    	$this->stripe = array(
  			"secret_key"      => $this->getConfigData('api_key'),
  			"publishable_key" => $this->getConfigData('publishable_api_key')
		);
		\Stripe\Stripe::setApiKey($this->stripe['secret_key']);
    }

    public function getStripePublishableKeys() {
    	return $this->stripe['publishable_key'];
    }
    
    public function assignData($data)
    {
        if (!($data instanceof Varien_Object)) {
            $data = new Varien_Object($data);
        }
        $info = $this->getInfoInstance();
        $session = Mage::getSingleton('checkout/session');
        $session->setPaymentToken($data->getToken());
        return $this;
    }

	public function authorize(Varien_Object $payment, $amount) {
	    // Mage::log('Starting payment authorize');
        $log = new Quivers_Integration_Model_LogTracking();
        $session = Mage::getSingleton('checkout/session');
        $payment_token = $session->getPaymentToken();
        $statementDescriptor = $session->getPaymentStatementDescriptor();
        
        $order = $payment->getOrder();
        $currency = strtolower($order->getBaseCurrencyCode());
        $chargeAmount = $this->formatCurrencyAmount($amount, $currency);

        $billing = $order->getBillingAddress();
        $customerEmail = $billing->getEmail();

        try {
            // get or create Stripe customer
            $customer = false;
            $customersList = \Stripe\Customer::all(array(
                "limit" => 1,
                "email" => $customerEmail,
            ));
            if(count($customersList["data"]) > 0) {
                $customer = $customersList["data"][0];
            }
            if (!$customer) {
                $customer = \Stripe\Customer::create(array(
                    'source' => $payment_token,
                    'email' => $customerEmail,
                ));
            }
            $customerId = $customer["id"];

            // add source to customer
            try {
                $customer->sources->create(array("source" => $payment_token));
            }
            catch (Exception $e) {}
            
            $charge = \Stripe\Charge::create(array(
                'customer' => $customerId,
                'source' => $payment_token,
                'capture' => false,
                'amount' => $chargeAmount,
                'currency' => strtolower($order->getBaseCurrencyCode()),
                'description' => sprintf('#%s, %s', $order->getIncrementId(), $order->getCustomerEmail()),
                'statement_descriptor_suffix' => $statementDescriptor,
            ));
        }
        catch (\Stripe\Error\Card $e) {
            Mage::throwException(Mage::helper('paygate')->__($e->getMessage()));
        }
        catch (Exception $e) {
            Mage::throwException(Mage::helper('paygate')->__('Payment capturing error.'));
        }

        $payment
            ->setTransactionId($charge->id)
            ->setPoNumber($charge->id)
            ->setIsTransactionClosed(0);

         if (Mage::helper('sales')->canSendNewOrderEmail($order->getStore()) &&
            $order->getEmailSent() != 1) {
                $order->sendNewOrderEmail();
        }
        $order_data = ["Customer Id"=>$customerId, "Email"=>$customerEmail];
        $response = ["Response"=>$charge, "Amount"=>$amount];
        $log->payment_request($order_data, "stripe", date('r', $charge["created"])."UTC", $response, $charge["status"]);
        return $this;
	}

    public function formatCurrencyAmount($amount, $currency)
    {

        // Stripe takes amounts as the smallest currency unit. ...

        if (empty($amount)) {
            return null;
        }
        $multiple = 100;
        if (in_array(
            $currency,
            array('BIF', 'DJF', 'JPY', 'KRW', 'PYG', 'VND', 'XAF', 'XPF', 'CLP', 'GNF', 'KMF', 'MGA', 'RWF', 'VUV', 'XOF'),
            true
        )) {
            $multiple = 1;
        }

        return sprintf('%01.0f', (float)$amount * $multiple);
    }

	public function isAvailable($quote = null)
    {
    	if($quote && $quote->getBaseGrandTotal()<$this->_minOrderTotal) {
    		return false;
    	}
    	
        $isAvailable =  $this->getConfigData('api_key', ($quote ? $quote->getStoreId() : null))
            && $this->getConfigData('active') == 1;
        return $isAvailable;
    }

    public function canUseForCurrency($currencyCode)
    {
        return true;
    }

    public function validate()
    {
        return true;
    }

}
