<?php

require_once BP.DS.'lib'.DS.'PayPal-PHP-SDK'.DS.'autoload.php';

class Quivers_Integration_Model_Paypalpayment extends Mage_Payment_Model_Method_Cc {

	protected $_code	=	'quivers_integration_paypal';
	protected $_formBlockType = 'quivers_integration/form_paypalpaymentmethod';
    // protected $_infoBlockType = 'quivers_integration/info_stripepaymentinfo';
	
    protected $_canAuthorize                = true;

    protected $clientId;
    protected $clientSecret;
    protected $sandbox;

	public function __construct()
    {
        $this->clientId = Mage::getStoreConfig('payment/quivers_integration_paypal/client_id');
        $this->clientSecret = Mage::getStoreConfig('payment/quivers_integration_paypal/client_secret');
        $this->sandbox = Mage::getStoreConfig('payment/quivers_integration_paypal/paypal_sandbox_enabled');
    }

    public function assignData($data)
    {
        if (!($data instanceof Varien_Object)) {
            $data = new Varien_Object($data);
        }
        $info = $this->getInfoInstance();
        $session = Mage::getSingleton('checkout/session');
        
        
        $session->setPaymentToken($data->getToken());
        $session->setPaymentPayer($data->getPayer());
        return $this;
    }

	public function authorize(Varien_Object $payment, $amount) {
		// Mage::log('Starting payment authorize');
		$startTime=microtime(true);
		$log = new Quivers_Integration_Model_LogTracking();
        $session = Mage::getSingleton('checkout/session');
        $paymentId = $session->getPaymentToken();
        $payerId = $session->getPaymentPayer();
        if(!$paymentId) {
            Mage::throwException(Mage::helper('paygate')->__('We are unable to recieve any payment details. Please try again.'));
        }
        
        $order = $payment->getOrder();
        $currency = $order->getBaseCurrencyCode();
        
        $chargeAmount = $amount;
        $apiContext = new \PayPal\Rest\ApiContext(
                new \PayPal\Auth\OAuthTokenCredential(
                    $this->clientId,     // ClientID
                    $this->clientSecret      // ClientSecret
                )
        );
        
        $mode = 'sandbox';
        if($this->sandbox != 1) {
            $mode = 'live';
        }

        $apiContext->setConfig(
              array(
                'mode' => $mode,
              )
        );
        
        try {
            $paypalPayment = \PayPal\Api\Payment::get($paymentId, $apiContext);
        } 
        catch(\PayPal\Exception\PayPalConnectionException $ex) {
            Mage::throwException(Mage::helper('paygate')->__('Connection Failed: Unable to retrieve payment. '.$ex->getData()));
        }
        catch (Exception $ex) {
            Mage::throwException(Mage::helper('paygate')->__('Unable to retrieve payment.'));
        }
        // ### Payment Execute
        // PaymentExecution object includes information necessary
        // to execute a PayPal account payment.
        $execution = new \PayPal\Api\PaymentExecution();
        $execution->setPayerId($payerId);
        // ### Optional Changes to Amount
        // If you wish to update the amount that you wish to charge the customer,
        // based on the shipping address or any other reason, you could
        // do that by passing the transaction object with just `amount` field in it.
        $transaction = new \PayPal\Api\Transaction();
        $paypalAmount = new \PayPal\Api\Amount();
        $paypalAmount->setCurrency($currency);
        $paypalAmount->setTotal($chargeAmount);
        $transaction->setAmount($paypalAmount);

        // Add the above transaction object inside our Execution object.
        $execution->addTransaction($transaction);
        try {
            $authorizationId = false;
            // Execute the payment
            $result = $paypalPayment->execute($execution, $apiContext);
            
            $authorizationId = $this->getAuthorizationId($result);
        }
        catch(\PayPal\Exception\PayPalConnectionException $ex) {
            Mage::throwException(Mage::helper('paygate')->__('Connection Failed: Unable to execute payment. '.$ex->getData()));
            } 
        catch (Exception $ex) {
            Mage::throwException(Mage::helper('paygate')->__('Unable to execute payment'));
        }
        if(!$authorizationId) {
            Mage::throwException(Mage::helper('paygate')->__('Unable to authorize payment.'));
        }

        $payment
            ->setTransactionId($authorizationId)
            ->setPoNumber($authorizationId)
            ->setIsTransactionClosed(0);
        $order_data = ["Amount"=>$paypalAmount->getTotal($chargeAmount)];
        $log->payment_request($order_data, "PayPal", date('r', $startTime)."UTC", ["Transaction Id"=>$authorizationId], "success");
        if (Mage::helper('sales')->canSendNewOrderEmail($order->getStore()) &&
            $order->getEmailSent() != 1) {
                $order->sendNewOrderEmail();
        }
        
        return $this;
	}

	public function isAvailable($quote = null)
    {
        $isAvailable =  $this->getConfigData('client_id', ($quote ? $quote->getStoreId() : null))
            && $this->getConfigData('active') == 1;
        return $isAvailable;
    }

    public function canUseForCurrency($currencyCode)
    {
        return true;
    }

    public function validate()
    {
        return true;
    }

    protected function getAuthorizationId($paypalPayment) {
        $transactions = $paypalPayment->getTransactions();
        foreach ($transactions as $transaction) {
            $relatedResources = $transaction->getRelatedResources();
            foreach ($relatedResources as $relatedResource) {
                $authorization = $relatedResource->getAuthorization();
                if($authorization) {
                    $authorizationId = $authorization->getId();
                    return $authorizationId;
                }
            }
        }

        return false;
    }

}
